#include "spooler.h"

int testExclusionMutua();
int testPrioridades();
int testPrioridades2();
int cliente(Spooler *sp, char *txt);
int clientesubmit(Spooler *sp, char *txt, int workdelay);

int nMain(int argc, char** argv) {
  // debugging= FALSE;

  testExclusionMutua();
  testPrioridades();
  testPrioridades2();

  return 0;
}

/* Test de exclusion mutua: 2 impresoras
 * t1 solicita job y comienza a imprimir
 * t2 solitica job y comienza a imprimir
 * t3 solicita job y espera
 * t1 termina de imprimir y comienza t3 a imprimir
 */
int testExclusionMutua() {
  nTask t1, t2, t3;
  Spooler *sp;

  nPrintf("\n");
  nPrintf("test de exclusion mutua\n");
  nPrintf("***********************\n\n");
  sp= makeSpooler(2);
  t1= nEmitTask(cliente, sp, "3000 exclusion mutua job 1");
  nSleep(1000);
  t2= nEmitTask(cliente, sp, "3000 exclusion mutua job 2");
  nSleep(1000);
  t3= nEmitTask(cliente, sp, "3000 exclusion mutua job 3");
  nWaitTask(t1); nWaitTask(t2); nWaitTask(t3);
  destroySpooler(sp);
  return 0;
}

/* Test de prioridad: 2 impresoras, los que solicitan con printJob tienen
 * prioridad con respecto a los que depositan con submitJob.
 * T=0: t1 solicita job y comienza a imprimir
 * T=1: t2 deposita job, retorna, el job comienza a imprimirse de inmediato
 * T=2: t3 deposita job, retorna, el job debe esperar
 * T=3: t4 solicita job y espera
 * T=4: se termina de imprimir el job de t2, t4 comienza a imprimir su job
 *      porque tiene prioridad sobre t3
 * T=5: t1 termina de imprimir, se comienza a imprimir job de t3
 * T=6: t3 pregunta por su job, no esta listo y espera
 * T=7: t2 pregunta por su job, esta listo y retorna
 * T=8: se termina de imprimir el job de t3 y retorna.
 * T=9: se termina de imprimir el job de t4
 */
int testPrioridades() {
  nTask t1, t2, t3, t4;
  Job *pjob1, *pjob2;
  Spooler *sp;
  nPrintf("\n");
  nPrintf("test de prioridades 1\n");
  nPrintf("*********************\n\n");
  sp= makeSpooler(2);
  t1= nEmitTask(cliente, sp, "5000 printJob t1");
  nSleep(1000);
  t2= nEmitTask(clientesubmit, sp, "3000 submitJob t2", 6000);
  nSleep(1000);
  t3= nEmitTask(clientesubmit, sp, "3000 submitJob t3", 4000);
  nSleep(1000);
  t4= nEmitTask(cliente, sp, "5000 printJob t4");
  nWaitTask(t1); nWaitTask(t2); nWaitTask(t3); nWaitTask(t4);
  destroySpooler(sp);
  return 0;
}

/* Test de prioridad 2: 2 impresoras, el que llama a waitJob retoma
 * la prioridad alta.
 * T=0: t1 deposita job, retorna, el job comienza a imprimirse en prt 0
 * T=1: t2 solitica job y comienza a imprimir en prt 1
 * T=2: t3 deposita job, retorna, el job debe esperar
 * T=3: t4 solicita job y espera
 * T=4: t3 pregunta por su job y espera
 * T=5: t5 solicita job y espera
 * T=6: prt 0 termina job t1, comienza a imprimir job t4
 * T=7: prt 1 termina job t2, comienza a imprimir job t3
 * T=8: t1 pregunta por su job y retorna
 * T=9: prt 1 termina job t3, comienza a imprimir job t5
 * T=10: prt 0 termina job t4
 * T=11: prt 1 termina job t5
 */
int testPrioridades2() {
  nTask t1, t2, t3, t4, t5;
  Job *pjob1, *pjob2;
  Spooler *sp;
  nPrintf("\n");
  nPrintf("test de prioridades 2\n");
  nPrintf("*********************\n\n");
  sp= makeSpooler(2);
  t1= nEmitTask(clientesubmit, sp, "6000 printJob t1", 8000);
  nSleep(1000);
  t2= nEmitTask(cliente, sp, "6000 submitJob t2");
  nSleep(1000);
  t3= nEmitTask(clientesubmit, sp, "2000 submitJob t3", 2000);
  nSleep(1000);
  t4= nEmitTask(cliente, sp, "4000 printJob t4");
  nSleep(2000);
  t5= nEmitTask(cliente, sp, "2000 printJob t5");
  nWaitTask(t1); nWaitTask(t2); nWaitTask(t3); nWaitTask(t4); nWaitTask(t5);
  destroySpooler(sp);
  return 0;
}

int cliente(Spooler *sp, char *txt) {
  nSetTaskName(txt);
  nPrintf("asking printJob %s\n", txt);
  printJob(sp, txt);
  nPrintf("done %s\n", txt);
  return 0;
}

int clientesubmit(Spooler *sp, char *txt, int workdelay) {
  Job *pjob;
  nSetTaskName(txt);
  nPrintf("asking submitJob %s\n", txt);
  pjob= submitJob(sp, txt);
  nPrintf("working without waiting %s\n", txt);
  nSleep(workdelay);
  nPrintf("waiting for %s\n", txt);
  waitJob(sp, pjob);
  nPrintf("done %s\n", txt);
  return 0;
}

void seqPrint(char *txt, int id) {
  char delayTxt[11];
  int i, delay;
  for (i= 0; i<10 && txt[i]>='0' && txt[i]<='9'; i++)
    delayTxt[i]= txt[i];
  delayTxt[i]= '\0';
  delay= atoi(delayTxt);
  nPrintf("printer %d: %s started, delay=%d millisecs\n", id, &txt[i+1], delay);
  nSleep(delay);
  nPrintf("printer %d: %s finished\n", id, &txt[i+1]);
}

